import numpy as np
import cv2
import pdb

cap = cv2.VideoCapture('isima1.mp4')
#cap = cv2.VideoCapture(0)

def drawpoints(frame,Pts):
    for pt in Pts:
        cv2.circle(frame,tuple(pt[0]),5,(0,0,255), thickness=2, lineType=8, shift=0)
    return frame    

# Load reference image
imgref = cv2.imread('ref_isima2.jpg')
# Create Detector and descriptor
orb = cv2.ORB_create(nfeatures=1500)
# Compute interest points and descriptors on the first image
kp_ref, ds_ref = orb.detectAndCompute(imgref,None)
while(True):
    # Capture frame-by-frame
    ret, frame = cap.read()
    if ret == False:
      break
    # Our operations on the frame come here
    gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    # Compute interest points and descriptors on the current frame
    kp_cur, ds_cur = orb.detectAndCompute(frame,None)
    # BFMatcher with default params
    # create BFMatcher object
    bf = cv2.BFMatcher(cv2.NORM_HAMMING, crossCheck=True)
    matches = bf.match(ds_ref,ds_cur)
    # Apply ratio test
    # Sort them in the order of their distance.
    matches = sorted(matches, key = lambda x:x.distance)
    # build src and dst pts
    f = 3 #threshold factor for inliers
    th = f * matches[0].distance
    n_matchs = 0
    for m in matches:
      if m.distance <= th:
        n_matchs = n_matchs +1
      else:
        break
    if n_matchs > 4: #at least 4 matches for homography estimation 
      src_pts = np.float32([kp_ref[m.queryIdx].pt for m in matches[0:n_matchs]]).reshape(-1, 1, 2)
      dst_pts = np.float32([kp_cur[m.trainIdx].pt for m in matches[0:n_matchs]]).reshape(-1, 1, 2)
      M, mask = cv2.findHomography(src_pts, dst_pts, cv2.RANSAC, 5.0)
      print('M = ',M)
      src_rect = np.array([[0, 0],
                           [imgref.shape[1],0],
                           [imgref.shape[1],imgref.shape[0]],
                           [0,imgref.shape[0]],
                           [0, 0]])
      cur_rect = cv2.perspectiveTransform(src_rect.reshape(-1,1,2).astype(np.float32), M)
      #pdb.set_trace()
      # displaynp.int0(hat)
      frame = cv2.polylines(frame, [np.int32(cur_rect)],True,255,3, cv2.LINE_AA)      

    # cv.drawMatchesKnn expects list of lists as matches.
    img3 = cv2.drawMatches(imgref,kp_ref,frame,kp_cur,matches[0:n_matchs],None,flags=cv2.DrawMatchesFlags_NOT_DRAW_SINGLE_POINTS)
    #pdb.set_trace()
    # Display the resulting frame
    cv2.imshow('ref image',imgref)
    cv2.imshow('color frame',frame)
    cv2.imshow('Matching result',img3)
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

# When everything done, release the capture
cap.release()
cv2.destroyAllWindows()